require 'uu/os/qos/qos_handler'
require 'uu/os/rest'
require 'uu/os/uesuri'
require 'uu/finman/businesscase/business_case_create'
require 'uu/finman/businesscase/business_case_attributes'
require 'uu/finman/businesscase/business_case_set_attributes'
require 'uu/finman/businesscase/business_case_set_state'
require 'uu/finman/businesscase/business_case_generate_report'

# Module Unicorn Universe.
module UU
  # Module Finman.
  module Finman
    # Module of finman Business case
    module BusinessCase
      extend self

      # Service path
      PATH = 'uu/finman/businesscase/BusinessCase'
      # Creates a new Business Case. The command creates a new Business Case
      # in a specified location  with passed DTO parameters.<br>
      #
      # <b>Required attributes:</b><br>
      #   meta_artifact_uri or business_case_type, name, business_case_company_uri, location_uri, currency
      # Attributes <code>default_account_uri</code> and <code>default_account_number</code>  cannot be set simultaneously.<br>
      # Attriburte <code>currency</code> must be one from allowed reporting currencies
      #
      # @param [String,UU::OS::UESURI] location_uri UESURI of the folder or organizational unit where Business Case will be created.
      # @param [UU::Finman::BusinessCase::BusinessCaseCreate] business_case DTO containing attributes of the new Business Case
      #
      # @return [UU::OS::UESURI] UESURI of the new Business Case. In case of setting <code>dry_run</code> parameter to <code>true</code>, <code>UU::OS::UESURI::NIL_URI</code> is returned instead.
      #
      #
      # @example
      #      # Prepare folder or Organization Unit URI, where we want to create Business Case.
      #      location_uri = UU::OS::UESURI.new('ues:TERRITORY:ARTIFACT_FOLDER_OR_OU');
      #
      #      # Create business Case
      #        UU::Finman::BusinessCase.create(location_uri,
      #        :meta_artifact_uri => "ues:TERRITORY:BUSINESS_CASE_META_ARTIFACT",
      #        :name => "Business case",
      #        :currency => "CZK",
      #        :business_case_company_uri => "ues:TERRITORY:ARTIFACT_COMPANY",
      #        :competent_role_uri => "ues:TERRITORY:COMPETENT_ROLE");
      #
      #      # Create business Case
      #        UU::Finman::BusinessCase.create(location_uri,
      #        :business_case_type => "INTRATERRITORIAL",
      #        :name => "Business case",
      #        :currency => "CZK",
      #        :business_case_company_uri => "ues:TERRITORY:ARTIFACT_COMPANY",
      #        :competent_role_uri => "ues:TERRITORY:COMPETENT_ROLE");
      def create(location_uri, business_case)
        svc = UU::OS::REST::RemoteClient.new(BusinessCase)
        bcase_create = UU::Finman::BusinessCase::BusinessCaseCreate.new(business_case).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('create', location_uri, bcase_create)
          return UU::OS::UESURI.new(res)
        end
      end

      # Obtains Business Case attributes from service and returns them
      #
      # @param [String, UU::OS::UESURI] business_case_uri UESURI of the Business Case
      #
      # @return [UU::Finman::BusinessCase::BusinessCaseAttributes] Business Case attributes
      #
      #
      # @example
      #      # Prepare Business Case URI, where we want to get attributes.
      #      business_case_uri = UU::OS::UESURI.new('ues:TERRITORY:BUSINESS_CASE');
      #
      #      # get business Case attributes
      #        UU::Finman::BusinessCase.get_attributes(business_case_uri);
      def get_attributes(business_case_uri)
        svc = UU::OS::REST::RemoteClient.new(BusinessCase)
        UU::OS::QoS::QoSHandler.auto_retry do
          attrs = svc.get('getAttributes', business_case_uri)
          return UU::Finman::BusinessCase::BusinessCaseAttributes.new(attrs)
        end
      end

      # Sets attributes of Business Case.
      #
      # @param [String,UU::OS::UESURI] business_case_uri UESURI of modified Business Case
      # @param [BusinessCaseSetAttributes] business_case_attributes DTO containing modified attributes of the Business Case
      #
      # @return [UU::OS::UESURI] UESURI of modified Business Case
      #
      #
      # @example
      #      # Prepare Business Case URI, where we want to set attributes.
      #      business_case_uri = UU::OS::UESURI.new('ues:TERRITORY:BUSINESS_CASE');
      #
      #      # set business Case attributes
      #        UU::Finman::BusinessCase.set_attributes(business_case_uri,
      #        :name => "Business case set name");
      def set_attributes(business_case_uri, business_case_attributes)
        svc = UU::OS::REST::RemoteClient.new(BusinessCase)
        bc_attrs = UU::Finman::BusinessCase::BusinessCaseSetAttributes.new(business_case_attributes).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('setAttributes', business_case_uri, bc_attrs)
          return UU::OS::UESURI.new(res)
        end
      end

      # Sets state of business case artifact.
      #
      # @param [String,UU::OS::UESURI] business_case_uri UESURI of modified Business Case
      # @param [BusinessCaseSetState] business_case_attributes DTO containing modified attributes of the Business Case
      #
      #
      # @example
      #      # Prepare Business Case URI, where we want to set state.
      #      business_case_uri = UU::OS::UESURI.new('ues:TERRITORY:BUSINESS_CASE');
      #
      #      # Set state of Business Case.
      #      UU::Finman::BusinessCase.set_state(business_case_uri, :business_case_state => "ACTIVE" ,:comment=>"Some sample comment");
      def set_state(business_case_uri, state)
        svc = UU::OS::REST::RemoteClient.new(BusinessCase)
        payload = UU::Finman::BusinessCase::BusinessCaseSetState.new(state).to_json
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('setState', business_case_uri, payload)
        end
      end
      
      
      # Sets default business case for organizational unit..
      #
      # @param [String,UU::OS::UESURI] business_case_uri UESURI of new default Business Case
      #
      # This command sets new default business case (main entity) for OU where the BC is. Former default BC becomes regular BC that is subordinate of the
      # new default BC. 
      # Updates befault attribute on primary satellite module and secondary satellite module. Recomputes assignments of payments (each payment is assigned to all superior default BCases)
      # to properly reflect the new default BC.
      # 
      # @example
      #      # Prepare Business Case URI, which should be the new default.
      #      business_case_uri = UU::OS::UESURI.new('ues:TERRITORY:BUSINESS_CASE');
      #
      #      # Set default Business Case.
      #      UU::Finman::BusinessCase.set_default(business_case_uri);
      def set_default(business_case_uri)
        svc = UU::OS::REST::RemoteClient.new(BusinessCase)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('setDefault', business_case_uri)
          return UU::OS::UESURI.new(res)
        end
      end

      # Generate report of Business Case.
      #
      # @param [String,UU::OS::UESURI] business_case_uri UESURI of modified Business Case
      # @param [BusinessCaseGenerateReport] attributes DTO containing attributes for generating report of the Business Case
      #
      # @return [UU::OS::UESURI] UESURI of created report attachment, or null UESURI if attachment was not generated.
      # If reportTypeAttachment and bcWithoutSubs and bcWithSubs is set to true, returns UESURI of created report attachment with subs.
      #
      #
      # @example
      #      # Prepare Business Case URI, where we want to set attributes.
      #      business_case_uri = UU::OS::UESURI.new('ues:TERRITORY:BUSINESS_CASE');
      #
      #      # generate report of business Case
      #        UU::Finman::BusinessCase.generate_report(business_case_uri,
      #        :dateFrom => "2017-01-01",
      #        :dateTo => "2017-12-31",
      #        :bcWithoutSubs => true,
      #        :bcWithSubs => false,
      #        :reportTypeContent => true,
      #        :reportTypeAttachment => true,
      #        :attachmentType => "PAY_DATE"
      # );
      def generate_report(business_case_uri, attributes)
        svc = UU::OS::REST::RemoteClient.new(BusinessCase)
        bc_attrs = UU::Finman::BusinessCase::BusinessCaseGenerateReport.new(attributes).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('generateReport', business_case_uri, bc_attrs)
          return UU::OS::UESURI.new(res)
        end
      end


      # Request update of BC structure on SSDB.
      #
      # @param [String,UU::OS::UESURI] control_panel_uri UESURI of Control Panel
      #
      # This command sends request to PSM to update BC structure table. PSM then computes BC structure and sends
      # request to SSM, where the structure will be stored.
      #
      # @example
      #      # Prepare Control Panel URI, which is in territory whose DB structure should be updated.
      #      control_panel_uri = UU::OS::UESURI.new('ues:TERRITORY:APPS.UU.FINMAN/CONTROL_PANEL');
      #
      #      # Request structure update.
      #      UU::Finman::BusinessCase.update_structure(control_panel_uri);
      def update_structure(control_panel_uri)
        svc = UU::OS::REST::RemoteClient.new(BusinessCase)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('updateStructure', control_panel_uri)
          return UU::OS::UESURI.new(res)
        end
      end
      
    end
  end
end
